import { performance } from 'perf_hooks';
import isPromise from '../../helpers/isPromise';
import isNode from '../../helpers/isNode';
import { statusCodes } from './statusCodes';
import { ConsoleReporter } from './reporters/ConsoleReporter';
export { DataDogReporter } from './reporters/DataDogReporter';

var getPerformanceMeter = function getPerformanceMeter() {
  return isNode ? performance : window.performance;
};

var getReporter = function getReporter() {
  var customReporter = isNode ? global.__SUI_DECORATOR_TRACER_REPORTER__ : window.__SUI_DECORATOR_TRACER_REPORTER__;
  return customReporter || new ConsoleReporter();
};

export default (function (_temp) {
  var _ref = _temp === void 0 ? {} : _temp,
      _ref$metric = _ref.metric,
      metric = _ref$metric === void 0 ? null : _ref$metric;

  return function (target, fnName, descriptor) {
    var configurable = descriptor.configurable,
        enumerable = descriptor.enumerable,
        writable = descriptor.writable;
    var originalGet = descriptor.get;
    var originalFunction = descriptor.value;
    var isGetter = !!originalGet; // https://github.com/jayphelps/core-decorators.js/blob/master/src/autobind.js

    return Object.assign({}, {
      configurable: configurable,
      enumerable: enumerable,
      get: function get() {
        var _this = this;

        var fn = isGetter ? originalGet.call(this) : originalFunction;

        if (this === target) {
          return fn;
        }

        var reporter = getReporter();
        var perf = getPerformanceMeter();
        var classMethodName = target.constructor.name + "::" + fnName;
        var metricName = metric || classMethodName;

        var _fnTimed = function _fnTimed() {
          // performance metric start
          var startTime = perf.now(); //  original function

          for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          var returnValue = fn.apply(_this, args);

          if (isPromise(returnValue)) {
            return returnValue.then(function (res) {
              var endTime = perf.now();

              if (res.__INLINE_ERROR__) {
                var error = res[0];
                reporter.send({
                  metricName: metricName,
                  status: error ? statusCodes.FAIL : statusCodes.SUCCESS,
                  value: endTime - startTime
                });
              } else {
                reporter.send({
                  metricName: metricName,
                  status: statusCodes.SUCCESS,
                  value: endTime - startTime
                });
              }

              return Promise.resolve(res);
            })["catch"](function (error) {
              var endTime = perf.now();
              reporter.send({
                metricName: metricName,
                status: statusCodes.FAIL,
                value: endTime - startTime
              });
              return Promise.reject(error);
            });
          } else {
            // performance metric ends
            var endTime = perf.now();
            reporter.send({
              metricName: metricName,
              status: statusCodes.SUCCESS,
              value: endTime - startTime
            });
            return returnValue;
          }
        };

        Object.defineProperty(this, fnName, {
          configurable: configurable,
          writable: writable,
          enumerable: enumerable,
          value: _fnTimed
        });
        return _fnTimed;
      },
      set: function set(newValue) {
        Object.defineProperty(this, fnName, {
          configurable: true,
          writable: true,
          enumerable: true,
          value: newValue
        });
        return newValue;
      }
    });
  };
});